@@ -1,9 +1,10 @@
 "trend.spatial" <-
   function (trend, geodata) 
 {
+  if(missing(geodata)) geodata <- list()
   if (inherits(trend, "formula")) {
     trend.frame <- geodata[attr(terms(trend), "term.labels")]
-    if(any(is.na(names(trend.frame)))){
+    if(any(is.null(names(trend.frame)) || is.na(names(trend.frame)))){
       trend.mat <- data.frame()
       class(trend.mat) <- "try-error"
     }
@@ -25,20 +26,23 @@
     }
   }
   else {
-    if (trend == "cte") 
+    if(is.numeric(trend))
+      trend.mat <- unclass(trend)
+    else if (trend == "cte") 
       trend.mat <- as.matrix(rep(1, nrow(geodata$coords)))
     else if (trend == "1st") 
       trend.mat <- cbind(1, geodata$coords)
     else if (trend == "2nd") 
       trend.mat <- cbind(1, geodata$coords, geodata$coords[,1]^2,
-                         geodata$coords[, 2]^2,
-                         geodata$coords[,1] * geodata$coords[, 2])
+                         geodata$coords[,2]^2,
+                         geodata$coords[,1] * geodata$coords[,2])
     ## This would use orthogonal polynomials:
     ##      trend.mat <- poly(as.matrix(geodata$coords), degree = 2)
     else stop("external trend must be provided for data locations to be estimated using the arguments trend.d and trend.l. Allowed values are \"cte\", \"1st\", \"2nd\" or  a model formula")
   }
   trend.mat <- as.matrix(trend.mat)
   dimnames(trend.mat) <- list(NULL, NULL)
+  class(trend.mat) <- "trend.spatial"
   return(trend.mat)
 }
 
@@ -419,7 +423,7 @@
   ##
   ## trend removal
   ##
-  xmat <- trend.spatial(trend = trend, geodata = x)
+  xmat <- unclass(trend.spatial(trend = trend, geodata = x))
   if (trend != "cte") {
     data <- lm(data ~ xmat + 0)$residuals
     names(data) <- NULL
