@@ -68,7 +68,7 @@
     method <- "ML"
   if(method == "ML" & cov.model == "power")
     stop("\n\"power\" model can only be used with method=\"RML\".\nBe sure that what you want is not \"powered.exponential\"")
-  xmat <- trend.spatial(trend=trend, geodata=geodata)
+  xmat <- unclass(trend.spatial(trend=trend, geodata=geodata))
   fit.ols <- lm(z ~ xmat + 0)
   trend.ols <- list(coefficients = fit.ols$coefficients)
   var.z <- sum((fit.ols$residual)^2)/(n-length(fit.ols$coefficients))
